/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : addr_latch.v
   Created on           : 19 Feb 1999
   Function             : Address latching for the target
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 
			22 feb 99	  Generates O_cfg_spc_enb bit 
		  	25 Feb 99	  Generates the O_single_cyc_xfer indication
            26 Feb 99	  Handles 64 bit data path requests                    
			02 Mar 99	  Signal names according to guidelines 
            15 Mar 99     do_disc port added 
            19 Mar 99     address parity error generation in this module 
            26 Mar 99     cbe added in the ports
            29 Apr 99     Blocking and non blocking assignments to the 
                          same signal resolved
   Instantiated in      : tgt_addr_ctl.v 

   Instantiates         :
   Synthesis check      : not done 
=========================================================================*/

/*	Whenever a new transfer is detected by the high to low transition on
	FRAME_n, the I_AD is latched to O_latched_addr_lo. If there is a dual
	address cyc indication by the target command decoder, I_AD is latched
	on to O_latched_addr_hi 
    I_REQ64_N is also latched at the same time as I_FRAME_N      

	The output O_first_xfer_cyc is a pulse which stays asserted for one 
	clock cyc, one clock cycle after the rising edge when AD(lower 32 bits) 
    gets latched.  The outout O_second_xfer_cyc is a pulse which stays 
    asserted for one clock cyc after the rising edge when AD gets latched 
    on to the higher 32 bits on a dual address cyc 

    The signal O_cfg_spc_enb gets asserted whenever AD[1:0] = 00 and 
    AD[10:8] = 000 ; These are necessary bu t not sufficient conditions 
    for generating the configuration register selects. This signal has 
    to be asserted and the decoder should decode the command to be a config
    command for the register selects to be generated 

    The signal O_single_cyc_xfer gets asserted for transfers that require 
    only one transfer cyc ( Frame asserted for one cyc only. This will
    be used for deciding whether to load the latched address to an 
    incrementor or directly pass it on. 

*/

module addr_latch (

					I_RST_N,				// From PCI bus
					I_CLK,
					I_FRAME_N,
                    I_REQ64_N,
					I_AD,
                    I_CBE,
                    I_par_from_pci,               

                    I_dac,		 			//  Dual address cyc 
                   
                    I_irdy_N ,				// directly from the bus
                    I_trdy_N ,

					O_latched_frame,		
					O_first_xfer_cyc,
					O_second_xfer_cyc,
                    O_cfg_spc_enb,

					O_single_cyc_xfer,		// added on 25th Feb
					O_latched_addr_lo,
					O_latched_addr_hi,

					O_req_64bit_xfer,
                    O_addr_par_err,
					O_do_disc

);


input			I_RST_N  ;
input			I_CLK ;
input			I_FRAME_N ;
input			I_REQ64_N ;
input	[31:0]	I_AD ;
input	[3:0]	I_CBE ;
input           I_par_from_pci ;


input 			I_dac ;
input 			I_irdy_N ;
input 			I_trdy_N ;

output	[31:0]	O_latched_addr_lo ;
output	[31:0]	O_latched_addr_hi ;

output 			O_latched_frame ;
output 			O_first_xfer_cyc ;
output 			O_second_xfer_cyc ;
output 			O_cfg_spc_enb ;
output 			O_single_cyc_xfer ;
output 			O_req_64bit_xfer ; // active high
output 			O_addr_par_err ; 
output          O_do_disc;

reg [31:0] latched_addr_lo ;
reg [31:0] latched_addr_hi ;
reg [3:0]  latched_cbe ;     // only for the address parity calculation

reg latched_frame ;
reg latched_req64 ;
reg frame_2 ; // state of frame 2 clocks before ; Useful for O_single_cyc_xfer
reg O_first_xfer_cyc ;
reg O_second_xfer_cyc ;
reg O_single_cyc_xfer ;
reg O_do_disc ;
reg recd_addr_prty ;
reg local_addr_prty ;

assign O_req_64bit_xfer = latched_req64 ; // use this in the target SM  
assign O_latched_frame = latched_frame ;
assign O_latched_addr_lo = latched_addr_lo ;
assign O_latched_addr_hi = latched_addr_hi ;

always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0) 
   begin 
      latched_frame <= 1'b1 ;  // de-asserted state of Frame 
      frame_2 <= 1'b1 ;
      latched_req64 <= 1'b0 ;
      
   end 
else
   begin 
      latched_frame <= I_FRAME_N ;
      frame_2 <= latched_frame ;

      latched_req64 <= ~I_REQ64_N ; // note the inversion
   end 
end 

always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0) 
   begin
      O_first_xfer_cyc <= 1'b0 ;
      O_second_xfer_cyc <= 1'b0 ;
      recd_addr_prty <= 1'b0 ;
      local_addr_prty <= 1'b0 ;
   end
else 
   begin 
// check conditions under which address parity has to be calculated for dual address cycles
      if ((latched_frame == 1'b0) && (frame_2 == 1'b1))
         begin 
            O_first_xfer_cyc <= 1'b1 ;
            recd_addr_prty <= I_par_from_pci ; // right time to look for address parity
            local_addr_prty <= calc_prty({latched_addr_lo,latched_cbe}) ; 
                         // prty on latched address and byte enables
         end 
      else   
         O_first_xfer_cyc <= 1'b0 ;

      if ((O_first_xfer_cyc == 1'b1) & (I_dac == 1'b1))
         O_second_xfer_cyc <= 1'b1 ;
      else
         O_second_xfer_cyc <= 1'b0 ;
   end  

end 

// latch the address on the bus during the address phase of a new PCI command

always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0) 

   begin
      latched_addr_lo <= 32'b0 ;
      latched_addr_hi <= 32'b0 ;
   end 

else  
   begin 
      if ((I_FRAME_N ==1'b0) & (latched_frame == 1'b1))
         begin 
            latched_addr_lo <= I_AD ;
            latched_cbe <= I_CBE ;
         end
      if ((O_first_xfer_cyc == 1'b1) & (I_dac))
         begin 
            latched_addr_hi <= I_AD ;
            latched_cbe <= I_CBE ;        // update only during address cycles
         end 
   end 
end          


always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0) 

   O_single_cyc_xfer <= 1'b0 ;
else
   begin 

      if ((I_FRAME_N == 1'b1) && (latched_frame == 1'b0) && (frame_2 == 1'b1))
         O_single_cyc_xfer <= 1'b1 ;  // set condition 

      if ((!I_irdy_N) && (!I_trdy_N))
         O_single_cyc_xfer <= 1'b0 ; // reset condition
   end 
end


// target indicates a disconnect wth data whenever a non-linear burst ordering 
// is found on the AD[1:0] 

always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0) 
   O_do_disc  <= 1'b0 ;
else
   if (O_first_xfer_cyc) 
      begin
      if (latched_addr_lo[1:0] != 2'b00) 
         O_do_disc  <= 1'b1 ;
      else
         O_do_disc  <= 1'b0 ;
      end
end 


// Probably for a single function PCI device, addr[10:8] need not be checked.
// Verify against the PCI spec

assign O_cfg_spc_enb = ~(latched_addr_lo[0] | latched_addr_lo[1] |
                            latched_addr_lo[10] | latched_addr_lo[9] |
                            latched_addr_lo[8] ) ;


assign O_addr_par_err = (local_addr_prty != recd_addr_prty);

function calc_prty ;
input [35:0] in_data ;  // check how to give a range 
begin

   calc_prty = ^in_data ;

end 
endfunction

endmodule
