/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : cfg_sm.v 
   Created on           : 5th March 99
   Function             : Generates the devsel,stop and trdy for config cycles
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : yes/no
   Modification History : 

   Instantiated in      : tgt_engine.v

   Instantiates         :
   Synthesis check      : done/not done 
=========================================================================*/

// This module generates the devsel, stop and trdy signals, only for config
// transactions. These signals for the mem & io transactions are generated
// in devsel_sm and tgt_sm modules

module cfg_sm (

					I_RST_N,
					I_CLK,

					I_first_xfer_cyc,	
					I_frame_N,		
                    I_irdy_N,
    
					I_valid_cfg_rd,	
					I_valid_cfg_wr,	

					O_devsel_N,
					O_oe_devsel, 	
					O_stop_N,		
					O_oe_stop,	
                    O_trdy_N,
					O_oe_trdy
    )	;


input   I_RST_N;
input   I_CLK;
input   I_first_xfer_cyc ;     
input   I_frame_N ; 
input   I_irdy_N ; 
input   I_valid_cfg_wr ;    
input   I_valid_cfg_rd ;    

output  O_devsel_N ;
output  O_oe_devsel ;
output  O_stop_N ;
output  O_oe_stop ;
output  O_trdy_N ;
output  O_oe_trdy ;

parameter cfg_idle = 2'b00 ;
parameter cfg_xfer = 2'b01 ;
parameter wait4frame = 2'b11 ; // wait for frame to get deasserted 
parameter last_cycle = 2'b10 ;
 
reg  O_devsel_N ;
reg  O_oe_devsel ;
reg  O_stop_N ;
reg  O_oe_stop ;
reg  O_trdy_N ;
reg  O_oe_trdy ;

reg [1:0] present_state ;
reg [1:0] next_state ;

wire      valid_cfg_transaction ;

assign valid_cfg_transaction = I_valid_cfg_rd | I_valid_cfg_wr  ;

always@(negedge I_RST_N or posedge I_CLK)
begin 
if (!I_RST_N)
   present_state <= cfg_idle ;
else
   present_state <= next_state ;
end 


always@(present_state)
begin
   case(present_state) 
   cfg_idle : 
      begin 
         O_devsel_N     <= 1'b1 ; // value does not matter
         O_stop_N       <= 1'b1 ; // de-asserted 
         O_trdy_N       <= 1'b1 ; // de-asserted 

	     O_oe_devsel    <= 1'b0 ; // All oes deactivated 
         O_oe_stop      <= 1'b0 ;  
	     O_oe_trdy      <= 1'b0 ; 
      end 

   cfg_xfer :
      begin 
         O_devsel_N     <= 1'b0 ; //  Assert all to indicate 
         O_stop_N       <= 1'b0 ; //  disconnect with single
         O_trdy_N       <= 1'b0 ; //  data 

	     O_oe_devsel    <= 1'b1 ; 
         O_oe_stop      <= 1'b1 ;  
	     O_oe_trdy      <= 1'b1 ; 

      end 

   wait4frame :
      begin 
         O_devsel_N     <= 1'b0 ; //  Keep asserting stop 
         O_stop_N       <= 1'b0 ; //  till frame  gets
         O_trdy_N       <= 1'b1 ; //  deasserted

	     O_oe_devsel    <= 1'b1 ; 
         O_oe_stop      <= 1'b1 ;  
	     O_oe_trdy      <= 1'b1 ; 


      end 

   last_cycle :
      begin 
         O_devsel_N     <= 1'b1 ; // Deassert all for 
         O_stop_N       <= 1'b1 ; // one cycle
         O_trdy_N       <= 1'b1 ; //

	     O_oe_devsel    <= 1'b1 ; // All oes deactivated 
         O_oe_stop      <= 1'b1 ;  
	     O_oe_trdy      <= 1'b1 ; 

      end 
   endcase 
end 



always@(present_state or I_first_xfer_cyc or valid_cfg_transaction or I_frame_N or I_irdy_N)
begin 

   case (present_state)
   
   cfg_idle :

      // New configuration transactions

      if ((I_first_xfer_cyc == 1'b1 ) && (valid_cfg_transaction == 1'b1))  
         next_state <= cfg_xfer ;
      else
         next_state <= cfg_idle ;

   cfg_xfer :

      if (!I_irdy_N ) // only when IRDY is assertes, the xfer will be over
         begin 
         if (I_frame_N)  // frame has already been deasserted 
            next_state <= last_cycle ;
         else 
            next_state <= wait4frame ;
         end
      else 
         next_state <= cfg_xfer ;
   
   wait4frame :
      if (I_frame_N)
         next_state <= last_cycle ;
      else
         next_state <= wait4frame ;

   last_cycle :
      next_state <= cfg_idle ;
   endcase 
end

endmodule 

