/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : devsel_sm.v
   Created on           : 19 Feb 1999
   Function             : Generates the DEVSEL signal for PCI interface 
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   26 Feb 99			  I_req_64 and I_capable64 signals 
                          added to the ports for 64 bit implementation
   6th March              Converted into a state machine
   20th March 
   24th Mar               Tristate at the proper time at the end of a xfer
   25th March             Corrected bug in DAC command transfers
   Instantiated in      : tgt_addr_ctl.v

   Instantiates         : no lower level modules
   Synthesis check      : not done 
=========================================================================*/

/*	This module generates 

    -- O_devsel_N  & O_ack64_N and their tri-state enables to claim a 
    transaction as a target for read and write transfers. These signals
    for the configuration cycles are generated by cfg_sm block
    
	-- O_trdy_N and O_stop_N signals with their tri-state enables for disconects
    and retrys happening during the first data phase of a transaction

	The O_stop_N and O_trdy_N generated here should be suitably combined 
    with the TRDY and STOP signals generated at the target sm and config sm 
    to go on the PCI bus.

	This module gets the latched address from the addr_latch block and 
	the base address register address from the configuration space ,
	and the decoded commands from the target command decoder. Matches 
    the addresses and generates a DEVSEL signal if an address match occurs 

	To indicate retry, when the data from previous transfer has not yet 
    reached the application from the FIFOs, a retry indication will have to be
    indicated during the first data phase. 

*/



module devsel_sm (

					I_RST_N,
					I_CLK,
					I_REQ64_N,			// direct from the bus 
					I_capable64,		// when 1, backend has 64 bit data path

					I_first_xfer_cyc,	
					I_second_xfer_cyc,
					I_frame_N,			    // latched version -- PROBABLY DIRECT FROM BUS
                       
					I_io_rd,				// From Target command decoder
                    I_io_wr,				//   		"
                    I_mem_rd,				//   		
                    I_mem_wr,				//   	
                    I_dac,            		// Dual address cycle         
					
                    I_lo_addr_match,
                    I_hi_addr_match,
                    I_io_addr_match,

					I_mem_spc_enb,			// From register bits
					I_io_spc_enb,

					I_end_xfer,					// From tgt_sm for indicating end of xfers
					I_back_end_busy,			// From the FIFO i/f ; indicates that 
												// there are transfers going on application 
												// i/f, So retry to be issued
					
					O_devsel_N,
					O_ack64_N,
					O_oe_devsel, 			// when '1', drive O_devsel_N value, else Z
					O_oe_ack64,				// when '1', drive O_devsel_N value, else Z
					O_stop_N,			    // This will be used for 
					O_oe_stop,			    // indicating target retry  or disconnect
                    O_trdy_N,
					O_oe_trdy,
                    O_tgt_rd,
                    O_tgt_wr,
					O_new_tgt_cmd			// Asserted for 1 clock cycle with every new
											// target command ( exept configs); This 
											// can be used to start the target sm 
    )	;

parameter idle_st = 2'b00 ;     // not driving any signals
parameter selected_st = 2'b01 ; // driving devsel and ack64 if required
parameter retry_st    = 2'b10 ; // driving devsel and stop and trdy 
parameter deassert_st = 2'b11 ; // driving all with de-asserted values


input		I_RST_N;
input		I_CLK;
input 		I_REQ64_N ;
input 		I_capable64 ;

input		I_first_xfer_cyc;		
input		I_second_xfer_cyc;		
input		I_frame_N;		

input		I_io_rd;	
input       I_io_wr;
input       I_mem_rd;			
input       I_mem_wr;		
input       I_dac; 

input 		I_lo_addr_match ; //asserted indicates low 4bytes address match
input 		I_hi_addr_match ; //asserted indicates high 4bytes address match
input 		I_io_addr_match ;


input 		I_io_spc_enb ;
input 		I_mem_spc_enb ;

input		I_back_end_busy ;
input		I_end_xfer ;  

output		O_stop_N ;
output		O_trdy_N ;
output		O_devsel_N ;
output		O_ack64_N ;
output		O_oe_devsel ;
output		O_oe_stop ;
output		O_oe_trdy ;
output		O_oe_ack64 ;
output 		O_tgt_rd ;
output 		O_tgt_wr ;

output		O_new_tgt_cmd ;

reg			O_devsel_N ;
reg			O_ack64_N ;
reg			O_oe_devsel ;
reg			O_oe_ack64 ;
reg			O_oe_stop ;
reg			O_oe_trdy ;
reg			O_stop_N ;
reg			O_trdy_N ;
reg 		new_tgt_cmd ;
reg			O_tgt_rd ;
reg			O_tgt_wr ;


reg			compare_64bit_addr ;
reg         latched_req64 ;

reg  [1:0]  present_state ;
reg  [1:0]  next_state ;

wire matched_64bit_addr ;
wire io_xfer ;       // indicates a I_io_rd or I_io_wr command
wire valid_io_xfer ; // indicated io_xfer with io_address match


wire mem_xfer ; // indicates a I_mem_rd or I_mem_wr command
wire valid_32_mem_xfer ; // indicates 32bit wide mem_xfer with mem_address match
wire valid_64_mem_xfer ; // indicates 64bit wide mem_xfer with mem_address match

assign matched_64bit_addr = I_lo_addr_match & I_hi_addr_match ;
assign O_new_tgt_cmd = new_tgt_cmd ;

// Generate only if target has io capability
assign io_xfer = (I_io_rd | I_io_wr) & I_io_spc_enb ;
assign valid_io_xfer = io_xfer & I_io_addr_match ;

// Generate only if target has memory space 
assign mem_xfer = (I_mem_rd | I_mem_wr) & I_mem_spc_enb  ;
assign valid_32_mem_xfer =  mem_xfer & I_lo_addr_match ;
assign valid_64_mem_xfer =  mem_xfer &  & I_lo_addr_match & latched_req64  & I_capable64 ; // CHECK IF hi address match should be considered or not


// for 32 bit address, 64 bit data ; for 64 bit address and 64bit data check later
// Will have to use matched_64bit_addr, compare_64bit_addr etc 

// latching the 64 bit data path request 

always@(posedge I_CLK or negedge I_RST_N) 
begin

if (!I_RST_N)
   latched_req64 <= 1'b0 ;
else
   begin
   if (I_end_xfer)
      latched_req64 <= 1'b0 ;
   else if (I_REQ64_N  == 1'b0)  
      latched_req64 <= 1'b1 ; // note the inversion
   end 
end


// state flip-flops 

always@(negedge I_RST_N or posedge I_CLK)

begin

if (I_RST_N == 1'b0) 
   present_state <= idle_st ;
else
   present_state <= next_state ;

end 


// Generate a flag for 64 bit address compare for DAC commands 
// Note : 64 bit address does not need REQ64 !!
// check if these conditions are sufficient to start a dac command
// May be the memory base address reg[2:1] indicating if the memory space is
// including 64 bit address space should be used as an enable condition

always@(negedge I_RST_N or posedge I_CLK)

begin

if (I_RST_N == 1'b0) 

   compare_64bit_addr <= 1'b0 ;
else 
   begin 
      if (I_dac == 1'b1)
         compare_64bit_addr <= 1'b1 ;
      else
         compare_64bit_addr <= 1'b0 ;
   end 
end 



// Generate O_tgt_rd whenever there is a read command intended for 
// this target (io or memory) Similarly O_tgt_wr. These two will go 
// to target state machine . It does not matter whether it is a 
// memory or io operation 

always@(negedge I_RST_N or posedge I_CLK)
begin

if (I_RST_N == 1'b0) 
   begin 
      O_tgt_rd <= 1'b0 ;
      O_tgt_wr <= 1'b0 ;
   end 
else 
   begin 

   if ((I_first_xfer_cyc == 1'b1) & (I_dac == 1'b0)) // SAC commands 

      begin 

         if  ( ((I_io_addr_match == 1'b1) & (I_io_rd == 1'b1))  |
               ((I_lo_addr_match == 1'b1) & (I_mem_rd == 1'b1))   ) 

            O_tgt_rd <= 1'b1 ;

         else if ( ((I_io_addr_match == 1'b1) & (I_io_wr == 1'b1)) |
                   ((I_lo_addr_match == 1'b1) & (I_mem_wr == 1'b1))   )

            O_tgt_wr <= 1'b1 ;

         else 
            begin 
               O_tgt_rd <= 1'b0 ;
               O_tgt_rd <= 1'b0 ;
            end  
      end 

/*   else if ((I_second_xfer_cyc == 1'b1) && (compare_64bit_addr == 1'b1) &&
(I_capable64 == 1'b1))  

64 bit data path capability not necessary for 64 bit addressing !!!
*/

// CHECK THIS CONDITION : SECOND XFER NOT COMING PROPERLY !!!

   else if ((I_second_xfer_cyc == 1'b1) && (compare_64bit_addr == 1'b1)) // 
   // have to check condition for 64 bit address space from regbits, do later   24 MAR 99
 

      begin 
      if ((I_lo_addr_match == 1'b1) & (I_hi_addr_match == 1'b1))
         begin  
            if (I_mem_rd == 1'b1) 
               O_tgt_rd <= 1'b1 ;
            else if (I_mem_wr == 1'b1)
               O_tgt_wr <= 1'b1 ; 
         end 
      else 
         begin 
            O_tgt_rd <= 1'b0 ;
            O_tgt_wr <= 1'b0 ;
         end 
      end 
    
   if (I_end_xfer == 1'b1) // end of  xfers when current target is selected

      begin
         O_tgt_wr <= 1'b0 ;
         O_tgt_rd <= 1'b0 ;

      end 
   end 
end 

// Output generation process  ; ACK64 should be driven only in 
// 64 bit case

//always@(present_state)
always@(present_state or I_end_xfer or valid_64_mem_xfer)
begin

   case(present_state)

   idle_st :
      begin

         O_devsel_N     <= 1'b1 ; // value does not matter
         O_stop_N       <= 1'b1 ; // de-asserted 
         O_ack64_N      <= 1'b1 ;  
         O_trdy_N       <= 1'b1 ; // de-asserted 
   
      	 O_oe_devsel    <= 1'b0 ; // All oes deactivated 
         O_oe_stop      <= 1'b0 ;  
      	 O_oe_ack64     <= 1'b0 ; 
      	 O_oe_trdy      <= 1'b0 ; 
   
      end 


   selected_st :
      begin

         O_devsel_N     <= 1'b0 ; // Devsel asserted and driven
     	 O_oe_devsel    <= 1'b1 ; // 

         O_stop_N       <= 1'b1 ; // de-asserted  and  not driven
         O_trdy_N       <= 1'b1 ; 
         O_oe_stop      <= 1'b0 ;  
      	 O_oe_trdy      <= 1'b0 ; 

         // if request for 64 bit memory transfers can be granted 
         // assert ack64 along with devsel 
         if (valid_64_mem_xfer)
            begin 
               O_ack64_N <= 1'b0 ;
               O_oe_ack64 <= 1'b1 ;
            end 
         else 
            begin 
               O_ack64_N <= 1'b1 ;
               O_oe_ack64 <= 1'b0 ;
            end 

      end 

   retry_st :  //assert stop and devsel withot trdy
      begin
         O_devsel_N     <= 1'b0 ; // Devsel asserted and driven
      	 O_oe_devsel    <= 1'b1 ; // 
         O_stop_N       <= 1'b0 ; // Stop asserted and driven
         O_oe_stop      <= 1'b1 ;  

         O_ack64_N      <= 1'b1 ; // de-asserted  and  not driven 
         O_trdy_N       <= 1'b1 ; 

      end 

   deassert_st : // Actually end of de-assertion of enables is done by
      begin      // the latched versions.(Some logic in top_glue does it)
                 // so oes need not be driven in the de-assert state 

         O_devsel_N     <= 1'b1 ; // All deasserted
         O_stop_N       <= 1'b1 ; // and actively driven
         O_ack64_N      <= 1'b1 ;  
         O_trdy_N       <= 1'b1 ; // de-asserted 
   
/*      	 O_oe_devsel    <= 1'b1 ; // All oes activated 
         O_oe_stop      <= 1'b1 ;  
      	 O_oe_ack64     <= 1'b1 ; 
      	 O_oe_trdy      <= 1'b1 ; 
*/
      	 O_oe_devsel    <= 1'b0 ; // All oes activated 
         O_oe_stop      <= 1'b0 ;  
      	 O_oe_ack64     <= 1'b0 ; 
      	 O_oe_trdy      <= 1'b0 ; 

   
      end 
   endcase
end 

always@(present_state or I_first_xfer_cyc or I_dac or valid_io_xfer or
        valid_32_mem_xfer or I_back_end_busy or I_second_xfer_cyc or 
        valid_64_mem_xfer or I_end_xfer or I_frame_N)
begin

   case(present_state)

   idle_st :

      if ((I_first_xfer_cyc == 1'b1) && (I_dac == 1'b0)) // SAC commands 

         if ((valid_io_xfer == 1'b1) || (valid_32_mem_xfer == 1'b1) )

            if (I_back_end_busy == 1'b1) 

               next_state <= retry_st ;
            else
               begin 
                  next_state <= selected_st ;     
               end   
         else 
            next_state <= idle_st ;

/*    DAC Cycles necessarily do not mean 64 bit mem xfer

      else if ((I_second_xfer_cyc == 1'b1) && (valid_64_mem_xfer == 1'b1))  
       // DAC,memory transactions with address match 
*/

      else if (I_second_xfer_cyc == 1'b1)  



         if (I_back_end_busy == 1'b1) 

            next_state <= retry_st ;
         else
            begin 
               next_state <= selected_st ;     
            end 

      else
         next_state <= idle_st ;

   selected_st :
      
      if (I_end_xfer)
         next_state <= deassert_st ;
      else
         next_state <= selected_st ;
        

   retry_st :
      if (I_frame_N)
         next_state <= deassert_st ;
      else // continue driving stop till frame is deasserted 
         next_state <= retry_st ;

   deassert_st :
      if (I_frame_N ) // frame deasserted 
         next_state <= idle_st ;
      else
         next_state <= deassert_st ;

   default :  // is this required ??
      next_state <= idle_st ;

   endcase 
end 

// generate the new_tgt_cmd flag for starting up the target state machine

always@(negedge I_RST_N or posedge I_CLK)

begin

if (I_RST_N == 1'b0) 
   new_tgt_cmd <= 1'b0 ;
else 
   if ((present_state == idle_st)  && (next_state == selected_st))  
      new_tgt_cmd <= 1'b1 ;
   else 
      new_tgt_cmd <= 1'b0 ;  // so will stay high for just 1 clock 
end 

endmodule 


