/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : pci_fifo.v
   Created on           : 10 March 99
   Function             : Fifo 16 word deep fifo 
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 
   This is built on the model given by Arvind Agraval

   Instantiated in      : 

   Instantiates         :
   Synthesis check      : done/not done 
=========================================================================*/

// File Name : pci_fifo.v
// Author           Edit Date
// Arvind Agrawal   03/08/99
// Description :
// --------------
// This is a 16 x 32 fifo. This module instantiates a 16 x 32 dual port ram
// and builds a wrapper on it to make a fifo.
//--------------------------------------------------------------------------

//`timescale 1ns / 10ps

module pci_fifo
   (
    I_data_in,
    I_we,
    I_re,
    I_pwrclr_N,
    I_fifo_reset, 
    O_full,
    O_empty,
    O_data_out,
    I_clk_wr,
    I_clk_rd
    );

`define true  1'b1
`define false 1'b0
`define TRUE  1'b1
`define FALSE 1'b0
   
   parameter		      datawidth             = 32;
   parameter		      memorydepth           = 16;
   parameter		      memorydepthbitpattern = 4'b1111;
   parameter		      depthpower            = 4;
   parameter		      dualport              = `true;
   
   input [(datawidth -1) :0]  I_data_in; // input data
   input		      I_we; // write enable
   input		      I_re; // read anable
   input		      I_pwrclr_N; // negative assreted
   input		      I_fifo_reset; // positive asserted
   input		      I_clk_wr; // Write clock
   input		      I_clk_rd; // Read clock
   output		      O_full; // signifies that the fifo in full
   output		      O_empty; // signifies that the fifo in empty
   output [(datawidth -1) :0] O_data_out; // output data
   
   // ********** Registers  **********
   reg			      full;
   reg			      empty;
   reg [(depthpower -1) : 0]  write_count;
   reg [(depthpower -1) : 0]  read_count;


   reg [(depthpower -1) : 0]  prev_wr_ptr;
   reg [(depthpower -1) : 0]  prev_rd_ptr;

   wire			      write_masked;
   wire			      read_masked;
   reg [(datawidth -1) : 0]   fifo_memory[0 : memorydepth-1];
  

   // KVR added these signals

   reg 	full_flag ;    // These will have to be synchronised 
   reg 	empty_flag ;
   reg  last_op_was_rd ;
   reg  last_op_was_wr ;
   reg reset_indi ; // stays at 1 till the very first write  
 
   assign O_empty = empty_flag; // not stretched
   assign O_full = full_flag;
   
   // ********** Initial **********
   // synopsys translate_off
   integer i;
   
   initial
      begin
//	 write_masked <= 0;  CHECK 
	 for (i = 0; i <= memorydepth-1 ; i = i+1)
	    fifo_memory[i] = 0;
	 //$display("%t - Setting all memory to zero\n", $realtime);
      end
   // synopsys translate_on
   

   assign read_masked  = I_re & ~empty_flag ;  // for dual port op      
   assign write_masked = I_we & ~full_flag ;     
   
   // ********** write_pointer **********
   always @(posedge I_clk_wr or negedge I_pwrclr_N)
      if (I_pwrclr_N == 1'b0 )
	 write_count <= memorydepthbitpattern;
      else if (I_fifo_reset == 1'b1)
	 write_count <= memorydepthbitpattern;
      else if (write_masked == 1'b1)
	 if (write_count > 0)
	    write_count <= write_count -1;
	 else
	    write_count <= memorydepthbitpattern;              
      else
	 write_count <= write_count;
   
   
   // ********** read_pointer **********
   always @(posedge I_clk_rd or negedge I_pwrclr_N)
      if (I_pwrclr_N == 1'b0 )
	 read_count <= memorydepthbitpattern;
      else if (I_fifo_reset == 1'b1)
	 read_count <= memorydepthbitpattern;
      else if (read_masked == 1'b1)
	 if (read_count > 0)
	    read_count <= read_count -1;
	 else
	    read_count <= memorydepthbitpattern;
      else
	 read_count <= read_count;
   
   
   // keep track of previous pointers 

   always@(posedge I_clk_wr or negedge I_pwrclr_N)
   begin
      if(I_pwrclr_N == 1'b0) 
         prev_wr_ptr <= 0 ;
      else
         prev_wr_ptr <= write_count ;

   end 

   always@(posedge I_clk_rd or negedge I_pwrclr_N)
   begin
      if(I_pwrclr_N == 1'b0) 
         prev_rd_ptr <= 0 ;
      else
         prev_rd_ptr <= read_count ;

   end 


   // keep track of last operation done 
   // Generate combinatorially 

   always@(write_count or read_count or prev_wr_ptr or prev_rd_ptr) 
   begin
      if (read_count != prev_rd_ptr) 
         begin
            last_op_was_rd = 1'b1 ;
            last_op_was_wr = 1'b0 ;
         end
      else if (write_count != prev_wr_ptr)
         begin 
            last_op_was_rd = 1'b0 ;
            last_op_was_wr = 1'b1 ;
         end
   end
            

always@(negedge I_pwrclr_N or posedge I_clk_wr)
begin

if (!I_pwrclr_N)

   reset_indi <= 1'b1 ;
else
   if (write_masked) // at least 1 write done 
      reset_indi <= 1'b0 ;
end

// Generate empty and full flags 

always@(read_count or write_count or I_pwrclr_N) 
begin 
   if (I_pwrclr_N == 1'b0)
      begin 
         full_flag <= 1'b0 ;
         empty_flag <= 1'b1 ;
      end 
   else

      begin
         if (read_count == write_count)
            begin

               if (last_op_was_wr) 
                  full_flag <= 1'b1 ;
               else 
                  full_flag <= 1'b0 ;

               if (reset_indi) 
   
                  empty_flag <= 1'b1 ;
   
               else if ((last_op_was_rd) && (!reset_indi)) // not before first write
                  empty_flag <= 1'b1 ;
               else 
                  empty_flag <= 1'b0 ;
            end
         else
            begin
               full_flag <= 1'b0 ;
               empty_flag <= 1'b0 ;
            end 
      end 

end 
 

/* the 16 x 32 ram is instantiated here. This ram is made of flip-flops and
 * can later be replaced by the Technology specific RAMs
 * ***************** */
   pci_ram16x32 u_pci_ram16x32
      (
       .I_rd_addr(read_count),
       .I_wr_addr(write_count),
       .I_wr_data(I_data_in),
       .O_rd_data(O_data_out),
       .I_we_N(~write_masked),  // needs active low
       .I_cs_N(1'b1),
       .I_pwrclr_N(I_pwrclr_N),
       .I_clk(I_clk_wr)
       );
   
   
endmodule //tgt_fifo


