/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_addr_ctl.v
   Created on           : 23 Feb 99
   Function             : Target addres controller
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no

   Modification History : 
   3rd Mar 99			  Target address comparator for selction instantiated
   4th Mar 99			  Interrupt register select added	
   5th Mar 99             I_irdy_N and I_trdy_N added to ports  ; 
                          used for single cycle xfer indication

   11th Mar 99            addr_gen instantiated ( to give target address on 
                          application interface)
                          I_app_clk added as input port

   15 Mar 99              AD[1:0] not being "00" will result in a disconnect 
                          with data as only linear burst ordering is supported.
                          A signal do_disc is generated for this purpose

   Instantiated in      : tgt_top.v

   Instantiates         :addr_latch.v
						 cfg_addr_decode.v
						 addr_comp.v
                         addr_gen.v
						
   Synthesis check      : not done 
=========================================================================*/

// This block provides the latched address for address incrementor
//                         address match & Validated config commands  
//                         Register selects during configuration 

module tgt_addr_ctl (
						I_RST_N,
						I_CLK,
						I_FRAME_N,
						I_REQ64_N,
                        I_AD,
                        I_CBE,
						I_IRDY_N, 
						I_trdy_N,
                        I_par_from_pci, 
						
						I_cfg_rd,			// from command decoder
						I_cfg_wr,
						I_dac,

						I_bar_mem_lo_addr,	// From config space
						I_bar_mem_hi_addr,	
						I_bar_io_addr,

                        I_incr_addr,
                        I_app_clk,
  		     
						O_valid_cfg_rd,     
						O_valid_cfg_wr,     

   		                O_device_reg_sel,
   		                O_cs_reg_sel,
   		                O_cc_reg_sel,
   		                O_lt_reg_sel,
   		                O_bar0_lo_reg_sel,
   		                O_bar0_hi_reg_sel,
	                 	O_bar1_reg_sel,
	                 	O_intr_reg_sel,

						O_latched_frame,
						O_req_64bit_xfer,
						O_first_xfer_cyc,  // will be used for address incrementing
						O_second_xfer_cyc, 
                        O_single_cyc_xfer,

                        O_addr_lo,
                        O_addr_hi,

						O_lo_addr_match	,
						O_hi_addr_match	,
						O_io_addr_match ,
                        O_addr_par_err,
                        O_do_disc
					
				);

input	I_RST_N ;
input	I_CLK ;
input	I_FRAME_N ;
input	I_REQ64_N ;
input	[31:0] I_AD ;
input   [3:0] I_CBE;
input 	I_IRDY_N ;
input 	I_trdy_N ;
input   I_par_from_pci;

input	I_cfg_rd ;
input	I_cfg_wr ;
input	I_dac ;

input 	[31:0] I_bar_mem_lo_addr ;
input 	[31:0] I_bar_mem_hi_addr ;
input 	[31:8] I_bar_io_addr ;

input   I_incr_addr;
input   I_app_clk;

output 	O_valid_cfg_rd ;    
output 	O_valid_cfg_wr ;     
output 	O_device_reg_sel ;
output 	O_cs_reg_sel ;
output 	O_cc_reg_sel ;
output 	O_lt_reg_sel ;
output 	O_bar0_lo_reg_sel ;
output 	O_bar0_hi_reg_sel ;
output 	O_bar1_reg_sel ;
output 	O_intr_reg_sel ;

output 		O_latched_frame ;
output 		O_req_64bit_xfer ;
output 		O_first_xfer_cyc ;
output 		O_second_xfer_cyc ;
output 		O_single_cyc_xfer ;

output [31:0] O_addr_lo;
output [31:0] O_addr_hi ;

output		O_lo_addr_match	;
output		O_hi_addr_match	;
output		O_io_addr_match ; 
output		O_addr_par_err ; 
output      O_do_disc ;

wire [31:0] latched_addr_lo ;
wire [31:0] latched_addr_hi ;

wire	cfg_spc_enb ;	// From addr_latch block 
wire [7:2] 	reg_addr;	// from O_latched_address_lo [7:2] 
wire    single_cyc_xfer ;
wire    first_xfer_cyc ;
wire    second_xfer_cyc ;
wire [63:0] addr_to_app ;

reg delayed_dac ;

always@(posedge I_CLK or negedge I_RST_N)
begin

if (!I_RST_N) 
   delayed_dac <= 1'b0 ;
else
   delayed_dac <= I_dac;  // will make it aligned with first_xfer_cyc
end 

assign reg_addr = latched_addr_lo [7:2]  ;
assign O_single_cyc_xfer = single_cyc_xfer ;
assign O_first_xfer_cyc = first_xfer_cyc ;
assign O_second_xfer_cyc = second_xfer_cyc ;

assign O_addr_lo[31:0] = addr_to_app [31:0] ;
assign O_addr_hi[31:0] = addr_to_app [63:32] ;

cfg_addr_decode  cfg_addr_decode (

                    .I_RST_N            (I_RST_N),
                    .I_CLK              (I_CLK),
					.I_cfg_rd 			(I_cfg_rd), 	
					.I_cfg_wr 			(I_cfg_wr),
					.I_cfg_spc_enb 		(cfg_spc_enb),

					.I_reg_addr 		(reg_addr), // relavent bits 
   	     
					.O_valid_cfg_rd 	(O_valid_cfg_rd),     
					.O_valid_cfg_wr 	(O_valid_cfg_wr),     
                    .O_device_reg_sel 	(O_device_reg_sel),
                    .O_cs_reg_sel 		(O_cs_reg_sel),
                    .O_cc_reg_sel 		(O_cc_reg_sel),
                    .O_lt_reg_sel 		(O_lt_reg_sel),
                    .O_bar0_lo_reg_sel 	(O_bar0_lo_reg_sel),
                    .O_bar0_hi_reg_sel 	(O_bar0_hi_reg_sel),
                    .O_bar1_reg_sel 	(O_bar1_reg_sel),
					.O_intr_reg_sel	    (O_intr_reg_sel)
);

addr_latch  addr_latch (

					.I_RST_N 			(I_RST_N),
					.I_CLK 				(I_CLK),
					.I_FRAME_N 			(I_FRAME_N) , // from PCI bus
                    .I_REQ64_N			(I_REQ64_N),  // from PCI bus
					.I_AD 				(I_AD),
                    .I_CBE              (I_CBE),
                    .I_par_from_pci     (I_par_from_pci),

                    .I_dac 				(I_dac),	
 	                .I_irdy_N           (I_IRDY_N), // should be directly from PCI bus
                    .I_trdy_N           (I_trdy_N),				

					.O_latched_frame	(O_latched_frame),                  

					.O_first_xfer_cyc 	(first_xfer_cyc), 
					.O_second_xfer_cyc 	(second_xfer_cyc),
                    .O_cfg_spc_enb 		(cfg_spc_enb),
					.O_single_cyc_xfer	(single_cyc_xfer),
					.O_latched_addr_lo 	(latched_addr_lo),
					.O_latched_addr_hi 	(latched_addr_hi),
					.O_req_64bit_xfer	(O_req_64bit_xfer),
                    .O_addr_par_err     (O_addr_par_err),
                    .O_do_disc          (O_do_disc)

);



addr_comp addr_comp ( 
					.I_CLK				(I_CLK),
					.I_RST_N			(I_RST_N),
					.I_latched_addr_lo	(latched_addr_lo),	
					.I_latched_addr_hi	(latched_addr_hi),
					.I_bar_mem_lo_addr	(I_bar_mem_lo_addr),	
					.I_bar_mem_hi_addr	(I_bar_mem_hi_addr),
					.I_bar_io_addr		(I_bar_io_addr),

					.O_lo_addr_match	(O_lo_addr_match),
					.O_hi_addr_match	(O_hi_addr_match),
					.O_io_addr_match	(O_io_addr_match)

); 


addr_gen addr_gen 

( 
                     .I_RST_N           (I_RST_N),
                     .I_CLK             (I_app_clk), // what will become to the loads !!
                     .I_single_cyc_xfer (single_cyc_xfer),
                     .I_first_xfer_cyc  (first_xfer_cyc),
                     .I_dac             (delayed_dac),
                     .incr_addr         (I_incr_addr),
                     .I_latched_addr_hi (latched_addr_hi), 
                     .I_latched_addr_lo (latched_addr_lo), 
                     .out_addr          (addr_to_app) 
) ;

endmodule 
