/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_cmd_decode.v
   Created on           : 19 Feb 1999
   Function             : Command decoder for the PCI target
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 
   22 Feb                 Changed the decoder outputs to be 
                          bits of a vector (code compaction)
   03 Mar				  Signal naming according to coding guidleinnes
   04 Mar 99		      Register IDSEL before using it for decode 
   Instantiated in      : 

   Instantiates         :
   Synthesis check      : done 
=========================================================================*/

/* This module keeps looking for a high to low transition on FRAME_N and 
   whenever it detects such a transition, registers the PCI command on
   CBE_n bits. The commads are decoded as per the following table 
   			
   The outputs are combinatorial. The address controller validates and 
   registers  them before giving it to the the target engine.


   CBE_n 

   0000		Reserved
   0001		Reserved
   0010		I/O read
   0011		I/O write
   0100		Reserved
   0101		Reserved
   0110		Memory Read
   0111		Memory Write
   1000		Reserved
   1001		Reserved
   1010		Configuration read
   1011		Configuration Write
   1100		Memory Read ( in place of memory read multiple)
   1101		Dual Address Cycle
   1110		Memory Read ( in place of memory read line)
   1111		Memory Write ( in place of memory write and invalidate)
*/

module tgt_cmd_decode (
						I_RST_N,
						I_CLK,
						I_FRAME_N,
						I_CBE_N,
						I_IDSEL,
                        
                        O_io_rd,
                        O_io_wr,
                        O_mem_rd,
                        O_mem_wr,
                        O_cfg_rd,
                        O_cfg_wr,
                        O_dac

 );


input		I_RST_N ;
input 		I_CLK ;
input 		I_FRAME_N ;
input [3:0]	I_CBE_N ;
input		I_IDSEL ;

output 		O_io_rd ;
output		O_io_wr ;
output		O_mem_rd ;
output		O_mem_wr ;
output		O_cfg_rd ;
output		O_cfg_wr ;

output      O_dac ;

reg [6:0] 	decoded_cmd ; 
reg			registered_idsel  ; // alon g with command

reg frame_1  ; // registers I_Frame  ( frame in previous cycle)
reg frame_2  ; // registered grame_1 ( Useful for dual address cycles) 
reg [3:0] registered_cmd ;

wire dac ;

assign O_dac    = dac ;

assign dac      = decoded_cmd[6] ;
assign O_cfg_wr = decoded_cmd[5] ;
assign O_cfg_rd = decoded_cmd[4] ;
assign O_mem_wr = decoded_cmd[3] ;
assign O_mem_rd = decoded_cmd[2] ;
assign O_io_wr = decoded_cmd[1] ;
assign O_io_rd = decoded_cmd[0] ;

 
always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0)
   begin 
      frame_1 <= 1'b0 ;
      frame_2 <= 1'b0 ;
      registered_cmd <= 4'b0000 ;
      registered_idsel <= 1'b0 ;
   end
else
   begin 
      frame_1  <= I_FRAME_N ;
      frame_2  <= frame_1 ;
      if (I_FRAME_N == 1'b0 & frame_1 == 1'b1 ) 
         begin 
            registered_cmd <= I_CBE_N ;
            registered_idsel <= I_IDSEL ;
         end 
//      if ((frame_1== 1'b0 & frame_2 == 1'b1 ) & (dac == 1'b1))    // dac output comes in next cycle !:-(((
     else if ((frame_1== 1'b0 & frame_2 == 1'b1 ) & (registered_cmd == 4'b1101)) 
        // for dual address cycles only
        // can I Merge this with previous if ??
         registered_cmd <= I_CBE_N ;
   end    
end 


always@(registered_cmd or registered_idsel)
begin
   case (registered_cmd)

      4'b0010 : 
		decoded_cmd = 7'b0000001 ;
   
      4'b0011 : 
		decoded_cmd = 7'b0000010 ;

      4'b0110,
      4'b1100,
      4'b1110 : 

		decoded_cmd = 7'b0000100 ;

      4'b0111,
      4'b1111 : 
    
		decoded_cmd = 7'b0001000 ;

      4'b1010 : 

        if (registered_idsel == 1'b1) // check this 
		   decoded_cmd = 7'b0010000 ;
        else
		   decoded_cmd = 7'b0000000 ;
      
      4'b1011 : 

        if (registered_idsel == 1'b1) // check this 
			decoded_cmd = 7'b0100000 ;
        else 
			decoded_cmd = 7'b0000000 ;

      4'b1101 : 

		decoded_cmd = 7'b1000000 ;
     
      default : 

		decoded_cmd = 7'b0000000 ;
   endcase  
end

endmodule 

